package proj3;

/**
 * A* gets the best possible path by always expanding on the node that yields the shortest path
 * It keeps track of the known path length from the start and estimated length to the end for every
 * node visited. This algorithm expands on the heap-based search.
 * 
 * The heuristic used to estimate the length to the end is the Manhattan distance: delta_x + delta_y
 */
public class AStarSearch extends HeapGraphSearch {
	
	@Override
	protected int heuristic(RoomNode start, RoomNode end) {
		//Manhattan distance
		return Math.abs(end.getX() - start.getX()) + Math.abs(end.getY() - start.getY());
	}
	
	@Override
	protected int newCost(RoomInfo n) { return n.cost + 1; }
	
	public String toString() { return "A* Search"; }
	public String getShortName() { return "A*"; }
}
