package proj3;

import java.util.Iterator;
import java.util.LinkedList;

import proj3.RoomNode.RoomState;

/**
 * Breadth first is a version of the generic graph search where the open list is kept as
 * a queue. This means that the algorithm will try all paths of length n before ever trying
 * a path of length n+1. As such, Breadth First guarantees a path with the smallest possible
 * node count. The drawback is that a great many nodes will be expanded.
 */
public class BreadthFirstSearch implements GraphSearch {
	private int expanded;
	
	public boolean search(RoomNode start, RoomNode end) {
		LinkedList<RoomNode> queue = new LinkedList<RoomNode>();
		expanded = 0;
		queue.offer(start);
		//start.setParent(null);
		RoomNode n;
		while(true) {
			if(queue.isEmpty())
				return false;
			n = queue.poll();
			expanded++;
			n.setState(RoomState.SEEN);
			if(n == end)
				break;
			Iterator<RoomNode> it = n.getNeighbors();
			while(it.hasNext()) {
				RoomNode child = it.next();
				if(child.getState() == RoomState.UNSEEN) {
					child.setState(RoomState.IN_OPEN_LIST);
					child.setParent(n);
					queue.offer(child);
				}
			}
		}
		while(n != start) {
			RoomNode parent = n.getParentPath();
			parent.setNextOnPath(n);
			parent.setState(RoomState.ON_PATH);
			n = parent;
		}
		end.setState(RoomState.ON_PATH);
		
		return true;
	}
	public int getLastExpandedNodeCount() { return expanded; }
	
	public String toString() { return "Breadth First Search"; }
	public String getShortName() { return "BFS"; }
}
