package proj3;

import java.util.Iterator;
import proj3.RoomNode.RoomState;

/**
 * Depth First search is a version of the generic graph search algorithm where the open
 * list is kept as a stack. This means that the algorithm will keep going along a path until
 * it hits a dead end, and then it will try a different path. There are no guarantees about the
 * path length. Since the algorithm is stack-based, we can utilize recursion to make coding easy.
 */
public class DepthFirstSearch implements GraphSearch {
	private int expanded;
	
	public int getLastExpandedNodeCount() { return expanded; }
	
	//The mark array keeps track of the parent nodes in the found path. This is like a cursor-style
	//way of adding a field to each RoomNode
	public boolean search(RoomNode start, RoomNode end) {
		if(start.getParentPath() == null) {
			expanded = 0; //Only resets the first time this is called (relies on precondition)
			start.setState(RoomState.SEEN);
		}
		if(start.equals(end)) {
			//We're there, return the empty path
			start.setState(RoomState.ON_PATH);
			return true;
		}
		
		//Go through all the adjacent nodes until we find a path
		Iterator<RoomNode> nodes = start.getNeighbors();
		while(nodes.hasNext()) {
			RoomNode n = nodes.next();
			if(n.getState() == RoomState.UNSEEN) { //If we haven't seen this node, recursively search it
				n.setState(RoomState.SEEN);
				n.setParent(start);
				expanded++;
				if(search(n, end)) { //If the search finds a path, we're done here
					start.setState(RoomState.ON_PATH);
					start.setNextOnPath(n); 
					return true;
				}
			}
		}
		return false;
	}
	
	public String toString() { return "Depth First Search"; }
	public String getShortName() { return "DFS"; }
}
