package proj3;

/**
 * GraphSearch provides a standard interface for the main application to use when path finding.
 * The brunt work is done in the search method.
 */
public interface GraphSearch {
	/**
	 * This method performs a path-find to get from the starting node to the ending node.
	 * Since the nodes contain all the information about their location and neighbors, the
	 * overall maze is not necessary. Each node contains references to a parent node and a
	 * nextOnPath node. This method updates these references to show the path. The parent of the
	 * start node will be null, and the nextOnPath for the end node will be null. There is a
	 * method in RoomNode to return a list that holds the entire path, RoomNode.getPath().
	 * 
	 * Precondition: All RoomNodes in the maze have their RoomState set to UNSEEN
	 * 
	 * @param start A RoomNode specifying the path beginning
	 * @param end A RoomNode specifying the goal
	 * @return True if a path was found.
	 * @see RoomNode#getPath()
	 */
	public boolean search(RoomNode start, RoomNode end);
	
	/**
	 * The algorithms are required to keep track of how many nodes are read from the open list. This
	 * is for algorithm performance comparison.
	 * 
	 * @return An int containing the number of nodes looked at in the previous search
	 */
	public int getLastExpandedNodeCount();
	
	/**
	 * This is used for displaying the algorithm in the status bar
	 */
	public String getShortName();
}
