package proj3;

import java.awt.Color;
import java.awt.SystemColor;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * RoomNode represents an individual room in the maze
 */
public class RoomNode {
	/**
	 * RoomState allows searching algorithms to mark nodes. It also helps the visualizer
	 * depict nodes.
	 */
	public enum RoomState {
		UNSEEN(' ',Color.WHITE),
		SEEN('#', new Color(100,150,255)),
		ON_PATH('.',SystemColor.control),
		IN_OPEN_LIST('?',Color.GREEN);
		
		private char c; //This char is the graphic to use when printing a room with this state
		private Color color;
		RoomState(char c, Color color) { this.c = c; this.color = color; }
		
		public char getMapChar() { return c; }
		public Color getColor() { return color; }
		public void setColor(Color c) { color = c; }
	}
	
	private int x,y; //z?
	private List<RoomNode> adjacent;
	private RoomState state;
	private RoomNode nextPath;
	private RoomNode parentPath;
	
	public RoomNode(int x, int y) {
		this(x,y, new LinkedList<RoomNode>());
	}
	public RoomNode(int x, int y, List<RoomNode> adjacent) {
		this.x = x;
		this.y = y;
		this.adjacent = adjacent;
		this.state = RoomState.UNSEEN;
		this.nextPath = null;
		this.parentPath = null;
	}
	
	public int getX() { return x; }
	public int getY() { return y; }
	public Iterator<RoomNode> getNeighbors() { return adjacent.iterator(); }
	public List<RoomNode> getPath() {
		RoomNode n = this;
		List<RoomNode> l = new LinkedList<RoomNode>();
		while(n != null) {
			l.add(n);
			n = n.getNextOnPath();
		}
		return l;
	}
	public RoomState getState() { return state; }
	public RoomNode getNextOnPath() { return nextPath; }
	public RoomNode getParentPath() { return parentPath; }
	public boolean isAdjacentTo(RoomNode n) { return adjacent.contains(n); }
	
	public void addNeighbor(RoomNode n) { adjacent.add(n); }
	public void setState(RoomState s) { state = s; }
	public void setNextOnPath(RoomNode n) { nextPath = n; }
	public void setParent(RoomNode n) { parentPath = n; }
	
	public boolean equals(RoomNode b) {
		return (x == b.getX()) && (y == b.getY());
	}
	
	public static void markNeighbors(RoomNode n1, RoomNode n2) {
		n1.addNeighbor(n2);
		n2.addNeighbor(n1);
	}
}
