package proj3.extras;

import java.util.Iterator;
import java.util.LinkedList;

import proj3.RoomNode;
import proj3.RoomNode.RoomState;

public class StepBreadthFirstSearch implements StepGraphSearch {
	private RoomNode start, end;
	private LinkedList<RoomNode> queue = new LinkedList<RoomNode>();
	private int expanded; 

	public boolean advance() {
		if(queue.isEmpty())
			return true;
		RoomNode n = queue.poll();
		expanded++;
		if(n == end) {
			end.setState(RoomState.ON_PATH);
			while(n != start) {
				RoomNode parent = n.getParentPath();
				parent.setNextOnPath(n);
				parent.setState(RoomState.ON_PATH);
				n = parent;
			}
			queue.clear();
			return true;
		}
		n.setState(RoomState.SEEN);
		Iterator<RoomNode> it = n.getNeighbors();
		while(it.hasNext()) {
			RoomNode child = it.next();
			if(child.getState() == RoomState.UNSEEN) {
				child.setState(RoomState.IN_OPEN_LIST);
				child.setParent(n);
				queue.offer(child);
			}
		}
		
		return false;
	}

	public void setStartAndEnd(RoomNode start, RoomNode end) {
		this.start = start;
		this.end = end;
		queue.clear();
		start.setState(RoomState.SEEN);
		queue.offer(start);
		expanded = 0;
	}

	public boolean search(RoomNode start, RoomNode end) {
		setStartAndEnd(start, end);
		while(!advance());
		return true;
	}

	public int getLastExpandedNodeCount() {
		return expanded;
	}
	
	public String toString() { return "Breadth First Search (Step by Step)"; }
	public String getShortName() { return "BFS (S)"; }

}
