package proj3.extras;

import java.util.Iterator;
import java.util.Stack;

import proj3.RoomNode;
import proj3.RoomNode.RoomState;

/**
 * This does end up with a slightly different effect then DepthFirstSearch
 * This is because using recursion is more "aggresive" than this true stack based
 * implementation. With recursion, the first neighbor is immediately recursed on.
 * Here, the stack is filled with all the neighbors before popping off the last one.
 */
public class StepDepthFirstSearch implements StepGraphSearch {
	private RoomNode start, end;
	private Stack<RoomNode> stack;
	private int expanded; 
	
	public StepDepthFirstSearch() {
		stack = new Stack<RoomNode>();
	}
	
	public boolean advance() {
		if(stack.isEmpty())
			return true; //Make sure search() exits
		RoomNode n = stack.pop();
		expanded++;
		if(n == end) {
			//Recover path
			n.setState(RoomState.ON_PATH);
			while(n != start) {
				RoomNode parent = n.getParentPath();
				parent.setNextOnPath(n);
				parent.setState(RoomState.ON_PATH);
				n = parent;
			}
			stack.clear();
			return true;
		}
		n.setState(RoomState.SEEN);
		Iterator<RoomNode> it = n.getNeighbors();
		while(it.hasNext()) {
			//We haven't found the path yet, so add all the neighbors to the path
			RoomNode child = it.next();
			if(child.getState() == RoomState.UNSEEN) {
				child.setState(RoomState.IN_OPEN_LIST);
				child.setParent(n);
				stack.push(child);
			}
		}
		return false;
	}
	
	public void setStartAndEnd(RoomNode start, RoomNode end) {
		this.start = start;
		this.end = end;
		stack.clear();
		start.setState(RoomState.SEEN);
		stack.push(start);
		expanded = 0;
	}
	
	public boolean search(RoomNode start, RoomNode end) {
		setStartAndEnd(start, end);
		while(!advance());
		return true;
	}

	public int getLastExpandedNodeCount() {
		return expanded;
	}
	
	public String toString() { return "Depth First Search (Step by Step)"; }
	public String getShortName() { return "DFS (S)"; }
}
