package view;

import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JPanel;

import proj3.Maze;
import proj3.RoomNode;

/**
 * This panel knows how to draw a particular maze. It will center the maze in the viewing area
 * if the area is too big.
 */
public class MazeView extends JPanel {
	private Maze m;
	
	public MazeView(Maze m) {
		setMaze(m);
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Dimension size = getSize();
		Dimension pref = getPreferredSize();
		if(size.width > pref.width) {
			g.translate((size.width - pref.width) / 2, 0);
		}
		if(size.height > pref.height) {
			g.translate(0, (size.height - pref.height) / 2);
		}
		g.translate(1,1);
		RoomNode[][] rooms = m.getRooms(); 
		/*for(RoomNode[] row : rooms) {
			for(RoomNode n : row) {
				RoomView.paintRoom(g,n);
			}
		}*/
		for(int y = 0; y < m.getHeight(); y++) {
			for(int x = 0; x < m.getWidth(); x++) {
				RoomNode n = rooms[x][y];
				int dbl = 0;
				if(x == 0 || rooms[x-1][y] == null) dbl |= RoomView.ROOMVIEW_WESTWALL;
				if(x == m.getWidth() - 1 || rooms[x+1][y] == null) dbl |= RoomView.ROOMVIEW_EASTWALL;
				if(y == 0 || rooms[x][y-1] == null) dbl |= RoomView.ROOMVIEW_NORTHWALL;
				if(y == m.getHeight() - 1 || rooms[x][y+1] == null) dbl |= RoomView.ROOMVIEW_SOUTHWALL;
				if(n != null)
					RoomView.paintRoom(g,n,dbl);
			}
		}
	}
	
	public Maze getMaze() { return m; }
	public void setMaze(Maze m) {
		this.m = m;
		Dimension size = new Dimension(m.getWidth()*RoomView.WIDTH+2,m.getHeight()*RoomView.HEIGHT+2);
		setSize(size); //This counteracts a wierd bug where the size is set just a little too big when packed
		setPreferredSize(size);
	}
}
