package view;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

import proj3.AStarSearch;
import proj3.BestFirstSearch;
import proj3.BreadthFirstSearch;
import proj3.DepthFirstSearch;
import proj3.GraphSearch;
import proj3.Maze;
import proj3.RoomNode.RoomState;
import proj3.extras.StepAStarSearch;
import proj3.extras.StepBestFirstSearch;
import proj3.extras.StepBreadthFirstSearch;
import proj3.extras.StepDepthFirstSearch;

public class MenuHandler implements ActionListener {
	static final public int MENUI_FILE = 0;
	static final public int MENUI_MAZE = 1;
	static final public int MENUI_VIEW = 2;
	static final public int MENUI_ALG = 3;
	
	static final private String MENU_FILE = "File";
	static final private String MENU_MAZE = "Maze";
	static final private String MENU_VIEW = "View";
	static final private String MENU_ALG = "Algorithm";

	static final private String MENU_FILE_RANDOM = "New Random Maze...";
	static final private String MENU_FILE_OPEN = "Open...";
	static final private String MENU_FILE_PRINT = "Print to File...";
	static final private String MENU_FILE_EXIT = "Exit";
	static final private String MENU_MAZE_NEXT = "Next Step";
	static final private String MENU_MAZE_GO = "Go";
	static final private String MENU_MAZE_RESET = "Reset";
	static final private String MENU_VIEW_SHOW_ROOM_COLORS = "Show Room Colors";
	static final private String MENU_VIEW_SHOW_CORNERS = "Show Corners";
	static final private String MENU_VIEW_EDIT_COLORS = "Edit Colors";
	static final private String MENU_ALG_DFS = "Depth First";
	static final private String MENU_ALG_BFS = "Breadth First";
	static final private String MENU_ALG_BEST = "Best First";
	static final private String MENU_ALG_ASTAR = "A*";
	static final private String MENU_ALG_STEP_DFS = "Step Depth First";
	static final private String MENU_ALG_STEP_BFS = "Step Breadth First";
	static final private String MENU_ALG_STEP_BEST = "Step Best First";
	static final private String MENU_ALG_STEP_ASTAR = "Step A*";

	static final private int MNEMONIC_FILE_RANDOM = KeyEvent.VK_N;
	static final private int MNEMONIC_FILE_OPEN = KeyEvent.VK_O;
	static final private int MNEMONIC_FILE_PRINT = KeyEvent.VK_P;
	static final private int MNEMONIC_FILE_EXIT = KeyEvent.VK_X; 
	static final private int MNEMONIC_MAZE_NEXT = KeyEvent.VK_N;
	static final private int MNEMONIC_MAZE_GO = KeyEvent.VK_G;
	static final private int MNEMONIC_MAZE_RESET = KeyEvent.VK_R;
	static final private int MNEMONIC_VIEW_SHOW_ROOM_COLORS = KeyEvent.VK_S;
	static final private int MNEMONIC_VIEW_SHOW_CORNERS = KeyEvent.VK_C;
	static final private int MNEMONIC_VIEW_EDIT_COLORS = KeyEvent.VK_E;
	static final private int MNEMONIC_ALG_DFS = KeyEvent.VK_D;
	static final private int MNEMONIC_ALG_BFS = KeyEvent.VK_B;
	static final private int MNEMONIC_ALG_BEST = KeyEvent.VK_E;
	static final private int MNEMONIC_ALG_ASTAR = KeyEvent.VK_A;
	static final private int MNEMONIC_ALG_STEP_DFS = KeyEvent.VK_E;
	static final private int MNEMONIC_ALG_STEP_BFS = KeyEvent.VK_R;
	static final private int MNEMONIC_ALG_STEP_BEST = KeyEvent.VK_T;
	static final private int MNEMONIC_ALG_STEP_ASTAR = KeyEvent.VK_S;
	
	private MazeRunnerWindow parent;
	
	public MenuHandler(MazeRunnerWindow parent) {
		this.parent = parent;
	}
	
	/**
	 * Constructs a menu to add to the window, using this class as an action listener
	 */
	public JMenuBar getMenu(GraphSearch gs) {
		JMenuBar menuBar = new JMenuBar();
		JMenu menu;
		JMenuItem item;
		
		//Build the file menu
		menu = new JMenu(MENU_FILE);
		item = new JMenuItem(MENU_FILE_RANDOM, MNEMONIC_FILE_RANDOM);
		item.addActionListener(this);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK));
		menu.add(item);
		item = new JMenuItem(MENU_FILE_OPEN, MNEMONIC_FILE_OPEN);
		item.addActionListener(this);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_DOWN_MASK));
		menu.add(item);
		item = new JMenuItem(MENU_FILE_PRINT, MNEMONIC_FILE_PRINT);
		item.addActionListener(this);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, KeyEvent.CTRL_DOWN_MASK));
		menu.add(item);
		menu.addSeparator();
		item = new JMenuItem(MENU_FILE_EXIT, MNEMONIC_FILE_EXIT);
		item.addActionListener(this);
		menu.add(item);
		menuBar.add(menu);
		
		//Build the Step-through menu
		menu = new JMenu(MENU_MAZE);
		item = new JMenuItem(MENU_MAZE_NEXT, MNEMONIC_MAZE_NEXT);
		item.addActionListener(this);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0));
		menu.add(item);
		item = new JMenuItem(MENU_MAZE_GO, MNEMONIC_MAZE_GO);
		item.addActionListener(this);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		menu.add(item);
		menu.addSeparator();
		item = new JMenuItem(MENU_MAZE_RESET, MNEMONIC_MAZE_RESET);
		item.addActionListener(this);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
		menu.add(item);
		menuBar.add(menu);
		
		//Build the View menu
		menu = new JMenu(MENU_VIEW);
		item = new JCheckBoxMenuItem(MENU_VIEW_SHOW_ROOM_COLORS);
		item.setMnemonic(MNEMONIC_VIEW_SHOW_ROOM_COLORS);
		item.setSelected(true);
		item.addActionListener(this);
		menu.add(item);
		item = new JMenuItem(MENU_VIEW_EDIT_COLORS);
		item.setMnemonic(MNEMONIC_VIEW_EDIT_COLORS);
		item.addActionListener(this);
		menu.add(item);
		item = new JCheckBoxMenuItem(MENU_VIEW_SHOW_CORNERS);
		item.setMnemonic(MNEMONIC_VIEW_SHOW_CORNERS);
		item.setSelected(true);
		item.addActionListener(this);
		menu.add(item);
		menuBar.add(menu);
		
		//Build the Algorithm selection menu
		menu = new JMenu(MENU_ALG);
		ButtonGroup group = new ButtonGroup();
		item = new JRadioButtonMenuItem(MENU_ALG_DFS);
		item.setMnemonic(MNEMONIC_ALG_DFS);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof DepthFirstSearch);
		group.add(item);
		menu.add(item);
		item = new JRadioButtonMenuItem(MENU_ALG_BFS);
		item.setMnemonic(MNEMONIC_ALG_BFS);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof BreadthFirstSearch);
		group.add(item);
		menu.add(item);
		item = new JRadioButtonMenuItem(MENU_ALG_BEST);
		item.setMnemonic(MNEMONIC_ALG_BEST);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof BestFirstSearch);
		group.add(item);
		menu.add(item);
		item = new JRadioButtonMenuItem(MENU_ALG_ASTAR);
		item.setMnemonic(MNEMONIC_ALG_ASTAR);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof AStarSearch);
		group.add(item);
		menu.add(item);
		menu.addSeparator();
		item = new JRadioButtonMenuItem(MENU_ALG_STEP_DFS);
		item.setMnemonic(MNEMONIC_ALG_STEP_DFS);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof StepDepthFirstSearch);
		group.add(item);
		menu.add(item);
		item = new JRadioButtonMenuItem(MENU_ALG_STEP_BFS);
		item.setMnemonic(MNEMONIC_ALG_STEP_BFS);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof StepBreadthFirstSearch);
		group.add(item);
		menu.add(item);
		item = new JRadioButtonMenuItem(MENU_ALG_STEP_BEST);
		item.setMnemonic(MNEMONIC_ALG_STEP_BEST);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof StepBestFirstSearch);
		group.add(item);
		menu.add(item);
		item = new JRadioButtonMenuItem(MENU_ALG_STEP_ASTAR);
		item.setMnemonic(MNEMONIC_ALG_STEP_ASTAR);
		item.addActionListener(this);
		item.setEnabled(true);
		item.setSelected(gs instanceof StepAStarSearch);
		group.add(item);
		menu.add(item);
		menuBar.add(menu);
		
		return menuBar;
	}
	
	public void actionPerformed(ActionEvent e) {
		JMenuItem source = (JMenuItem)e.getSource();
		String action = source.getText();
		if(action.equals(MENU_FILE_OPEN)) { //Open a maze from a file
			parent.openMaze();
		} else if(action.equals(MENU_FILE_RANDOM)) { //Creates a new random maze with user-specified parameters
			Maze m = parent.getMaze();
			JFormattedTextField txtWidth = new JFormattedTextField();
			JFormattedTextField txtHeight = new JFormattedTextField();
			JFormattedTextField txtCycles = new JFormattedTextField();
			txtWidth.setValue(m.getWidth());
			txtHeight.setValue(m.getHeight());
			txtCycles.setValue(0);
			Object[] objects = {"Width:", txtWidth,
					"Height:", txtHeight,
					"Cycles:", txtCycles};
			JOptionPane mazeDialog = new JOptionPane(objects, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
			
			mazeDialog.createDialog(null, "New Random Maze").setVisible(true);
			if(((Integer)mazeDialog.getValue()) == JOptionPane.OK_OPTION) {
				int width = Integer.parseInt(txtWidth.getText());
				int height = Integer.parseInt(txtHeight.getText());
				int cycles = Integer.parseInt(txtCycles.getText());
				parent.randomMaze(width, height, cycles);
			}
		} else if(action.equals(MENU_FILE_PRINT)) { //Print the maze and the state of the nodes to a file (like the -final arg)
			parent.printMaze();
		} else if(action.equals(MENU_FILE_EXIT)) { //Quit
			parent.quit();
		} else if(action.equals(MENU_MAZE_NEXT)) { //Advance the search algorithm
			parent.advanceSearch();
		} else if(action.equals(MENU_MAZE_GO)) { //Finish the search algorithm
			parent.finishSearch();
		} else if(action.equals(MENU_MAZE_RESET)) { //Reset the state of the rooms
			parent.resetMaze();
		} else if(action.equals(MENU_VIEW_SHOW_ROOM_COLORS)) { //Viewing preferences
			parent.setShowRoomColors(source.isSelected());
		} else if(action.equals(MENU_VIEW_EDIT_COLORS)) {
			ColorButton seen = new ColorButton(RoomState.SEEN.getColor());
			ColorButton unseen = new ColorButton(RoomState.UNSEEN.getColor());
			ColorButton path = new ColorButton(RoomState.ON_PATH.getColor());
			ColorButton open = new ColorButton(RoomState.IN_OPEN_LIST.getColor());
			Object[] objects = {"Seen:", seen,
								"Unseen:", unseen,
								"On Path:", path,
								"Open List:", open};
			JOptionPane editColorDialog = new JOptionPane(objects, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
			
			editColorDialog.createDialog(null, "Edit Colors").setVisible(true);
			if(((Integer)editColorDialog.getValue()) == JOptionPane.OK_OPTION) {
				RoomState.SEEN.setColor(seen.getColor());
				RoomState.UNSEEN.setColor(unseen.getColor());
				RoomState.ON_PATH.setColor(path.getColor());
				RoomState.IN_OPEN_LIST.setColor(open.getColor());
				parent.repaint();
			}
		} else if(action.equals(MENU_VIEW_SHOW_CORNERS)) {
			parent.setShowCorners(source.isSelected());
		} else if(action.equals(MENU_ALG_DFS)) { //Select the algorithm
			parent.setAlgorithm(new DepthFirstSearch());
		} else if(action.equals(MENU_ALG_BFS)) {
			parent.setAlgorithm(new BreadthFirstSearch());
		} else if(action.equals(MENU_ALG_BEST)) {
			parent.setAlgorithm(new BestFirstSearch());
		} else if(action.equals(MENU_ALG_ASTAR)) {
			parent.setAlgorithm(new AStarSearch());
		} else if(action.equals(MENU_ALG_STEP_DFS)) {
			parent.setAlgorithm(new StepDepthFirstSearch());
		} else if(action.equals(MENU_ALG_STEP_BFS)) {
			parent.setAlgorithm(new StepBreadthFirstSearch());
		} else if(action.equals(MENU_ALG_STEP_BEST)) {
			parent.setAlgorithm(new StepBestFirstSearch());
		} else if(action.equals(MENU_ALG_STEP_ASTAR)) {
			parent.setAlgorithm(new StepAStarSearch());
		}
	}
	
	/**
	 * This class is used by the Edit Colors dialog
	 */
	private class ColorButton extends JButton implements ActionListener {
		private Color color;
		
		public ColorButton(Color c) {
			super("   ");
			color = c;
			setBackground(color);
			addActionListener(this);
		}
		
		public Color getColor() { return color; }
		
		public void actionPerformed(ActionEvent e) {
			color = JColorChooser.showDialog(null, "Pick Color", color);
			setBackground(color);
		}
	}
}
